@@ -176,7 +176,7 @@ panel_applet_locked_change_notify (GConfClient *client,
 		return;
 
 	value = gconf_entry_get_value (entry);
-	if (value->type != GCONF_VALUE_BOOL)
+	if (value == NULL || value->type != GCONF_VALUE_BOOL)
 		return;
 
 	locked = gconf_value_get_bool (value);
@@ -512,7 +512,7 @@ panel_applet_create_menu (AppletInfo *info)
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (applet_remove_callback), info);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-		gtk_widget_set_sensitive (menuitem, removable);
+		gtk_widget_set_sensitive (menuitem, (!locked || lockable) && removable);
 		
 		menuitem = gtk_image_menu_item_new ();
 		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, NULL, _("_Move"), FALSE);
@@ -603,7 +603,8 @@ panel_applet_position_menu (GtkMenu   *menu,
 	}
 
 	if (PANEL_WIDGET (applet->parent)->orient == GTK_ORIENTATION_HORIZONTAL) {
-		if (requisition.width < pointer_x)
+		if (pointer_x < applet->allocation.width &&
+		    requisition.width < pointer_x)
 			menu_x += MIN (pointer_x, applet->allocation.width - requisition.width);
 		menu_x = MIN (menu_x, gdk_screen_get_width (screen) - requisition.width);
 
@@ -612,7 +613,8 @@ panel_applet_position_menu (GtkMenu   *menu,
 		else
 			menu_y += applet->allocation.height;
 	} else {
-		if (requisition.height < pointer_y)
+		if (pointer_y < applet->allocation.height &&
+		    requisition.height < pointer_y)
 			menu_y += MIN (pointer_y, applet->allocation.height - requisition.height);
 		menu_y = MIN (menu_y, gdk_screen_get_height (screen) - requisition.height);
 
