@@ -1031,16 +1031,15 @@ panel_menu_button_set_dnd_enabled (PanelMenuButton *button,
 		static GtkTargetEntry dnd_targets [] = {
 			{ "application/x-panel-applet-internal", 0, 0 }
 		};
+		char *icon;
 
 		GTK_WIDGET_UNSET_FLAGS (button, GTK_NO_WINDOW);
 		gtk_drag_source_set (GTK_WIDGET (button), GDK_BUTTON1_MASK,
 				     dnd_targets, 1,
 				     GDK_ACTION_COPY | GDK_ACTION_MOVE);
-		/* FIXME: waiting for bug #116577
 		icon = panel_menu_button_get_icon (button);
 		gtk_drag_source_set_icon_name (GTK_WIDGET (button), icon);
 		g_free (icon);
-		*/
 		GTK_WIDGET_SET_FLAGS (button, GTK_NO_WINDOW);
 	} else
 		gtk_drag_source_unset (GTK_WIDGET (button));
