@@ -176,9 +176,12 @@ panel_run_dialog_set_icon (PanelRunDialog *dialog,
 
 	if (icon_path)
 		pixbuf = panel_load_icon (gtk_icon_theme_get_default (),
-					  icon_path, ICON_SIZE, NULL);
-
-                        
+					  icon_path,
+					  ICON_SIZE,
+					  ICON_SIZE,
+					  ICON_SIZE,
+					  NULL);
+	
 	if (pixbuf) {
 		dialog->icon_path = g_strdup (icon_path);
 
@@ -199,8 +202,8 @@ panel_run_dialog_set_icon (PanelRunDialog *dialog,
 		gtk_window_set_icon_name (GTK_WINDOW (dialog->run_dialog),
 					  PANEL_RUN_ICON);
 		
-		/* FIXME: waiting for bug #116577
-		gtk_drag_source_set_icon_name (dialog->run_dialog, PANEL_RUN_ICON); */
+		gtk_drag_source_set_icon_name (dialog->run_dialog,
+					       PANEL_RUN_ICON);
 	}
 }
 
@@ -374,8 +377,8 @@ panel_run_dialog_execute (PanelRunDialog *dialog)
 	scheme = gnome_vfs_get_uri_scheme (url);
 	result = FALSE;
 	
-	if (!strcasecmp (scheme, "http") ||
-	    !strcasecmp (scheme, "file"))
+	if (!g_ascii_strcasecmp (scheme, "http") ||
+	    !g_ascii_strcasecmp (scheme, "file"))
 		/* If this returns an http or file url, the url might refer to a
 		 * command that is somewhere in the path or an executable file.
 		 * So try executing it before displaying it. We execute the 
@@ -496,6 +499,8 @@ fuzzy_command_match (const char *cmd1,
 	char **tokens;
 	char  *word1, *word2;
 
+	g_return_val_if_fail (cmd1 && cmd2, TRUE);
+
 	*fuzzy = FALSE;
 
 	if (!strcmp (cmd1, cmd2))
@@ -790,15 +795,17 @@ panel_run_dialog_add_items_idle (PanelRunDialog *dialog)
 	prev_name = NULL;
 	for (l = all_applications; l; l = next) {
 		GMenuTreeEntry *entry = l->data;
+		const char     *entry_name;
 
 		next = l->next;
 
-		if (prev_name && strcmp (gmenu_tree_entry_get_name (entry), prev_name) == 0) {
+		entry_name = gmenu_tree_entry_get_name (entry);
+		if (prev_name && entry_name && strcmp (entry_name, prev_name) == 0) {
 			gmenu_tree_item_unref (entry);
 
 			all_applications = g_slist_delete_link (all_applications, l);
 		} else {
-			prev_name = gmenu_tree_entry_get_name (entry);
+			prev_name = entry_name;
 		}
 	}
 
@@ -1601,10 +1608,10 @@ pixmap_drag_data_get (GtkWidget          *run_dialog,
 						       GNOME_VFS_MAKE_URI_DIR_HOMEDIR);
 	scheme = gnome_vfs_get_uri_scheme (uri);
 	
-	if (!strcasecmp (scheme, "http"))
+	if (!g_ascii_strcasecmp (scheme, "http"))
 		exec = command_is_executable (text);
 		
-	else if (!strcasecmp (scheme, "file"))
+	else if (!g_ascii_strcasecmp (scheme, "file"))
 		exec = command_is_executable (uri);
 		
 	if (exec) {
